import posixpath
import os
from jinja2 import (
    Environment,
    FileSystemLoader
)
from jinja2.loaders import split_template_path
from jinja2.exceptions import TemplateNotFound
import typing as t
from pathlib import Path
import re


def replace_md_resource_paths(content: str, relative_dir: str):
    """查找并替换content(markdown格式)文本中资源引用路径
    目前处理以下2种类型的资源引用：
    - 链接： [alt text](./foo/bar.pdf)
    - 链接： [alt text](foo/bar.pdf)
    - 图片： ![alt text(./foo/bar.png)]
    - 图片： ![alt text(foo/bar.png)]

    使用将上述以相对路径引用的本地资源路径，替换为添加relative_dir_path目录前缀的路径
    例如：如果relative_dir_path = test，则上面的资源引用路径将被替换成：
    - 链接： [alt text](test/foo/bar.pdf)
    - 链接： [alt text](test/foo/bar.pdf)
    - 图片： ![alt text(test/foo/bar.png)]
    - 图片： ![alt text(test/foo/bar.png)]

    注意：仅替换引用本地资源的路径。如果引用的资源路径是https://或http://开头的url，则不进行替换
    """
    # 确保 relative_dir 没有前导或尾随的斜杠，以便正确拼接路径
    relative_dir = relative_dir.strip('/').strip('\\')

    # 正则表达式解释:
    # 1. r'(!?\[.*?\]\()'：
    #    - `!?`: 匹配0或1个`!`，用于区分图片和链接。
    #    - `\[.*?\]`: 匹配`[...]`中的任意非贪婪内容，即alt text。
    #    - `\(`: 匹配链接路径前的左括号。
    #    - 整个部分被包裹在圆括号里，形成第一个捕获组(Group 1)。
    # 2. `(.+?)`:
    #    - 匹配括号内的路径，`+`确保路径非空，`?`使其非贪婪。
    #    - 这是第二个捕获组(Group 2)，也是我们感兴趣的路径部分。
    # 3. `\)`:
    #    - 匹配路径后的右括号，这是第三个捕获组(Group 3)。
    # re.DOTALL 标志让 . 能匹配换行符，以便处理多行Markdown。
    pattern = re.compile(r'(!?\[.*?\]\()(.+?)(\))', re.DOTALL)

    def replacer(match):
        """
        这个内部函数作为 re.sub() 的回调函数，
        用于对每个匹配到的路径进行逻辑判断和替换。
        """
        # 获取捕获组中的路径
        path = match.group(2)

        # 1. 检查是否为绝对URL，如果是则不替换
        if path.startswith(('http://', 'https://', 'ftp://')):
            return match.group(0)  # 返回原始匹配的完整字符串

        # 2. 如果路径以 "./" 开头，去除它
        if path.startswith('./'):
            path = path[2:]

        # 3. 拼接新的路径，并确保使用正确的斜杠
        new_path = os.path.join(relative_dir, path).replace(os.sep, '/')

        # 4. 重新组合替换后的字符串
        prefix = match.group(1)  # ![alt text]( 或 [alt text](
        suffix = match.group(3)  # )
        return f"{prefix}{new_path}{suffix}"

    # 使用 re.sub() 和 replacer 函数进行替换
    return pattern.sub(replacer, content)


class MarkdownFileSystemLoader(FileSystemLoader):
    def __init__(self, *args, **kwargs):
        self.entry_file_path = None
        super().__init__(*args, **kwargs)

    def process_content(self, entry_file_path: Path, current_file_path: Path):
        # NOTE：文件与静态图片之间的依赖关系由ninja_cli加入到build.ninja中，不在这处理
        # 参考:make_ninja_build phony_static_res
        with open(current_file_path, encoding=self.encoding) as f:
            # 保存当前处理的文件路径，去除掉searchpath前缀，即去除input_dir前缀
            content = f.read()
            if entry_file_path == current_file_path:
                return content
            # 取得当前文件路径相对于入口路径的目录路径
            relative_dir = Path(os.path.relpath(
                current_file_path, entry_file_path.parent)).parent.as_posix()
            # 如果是同一个目录，则不需要替换
            if relative_dir == '.':
                return content
            # 否则将资源引用路径替换为带相对目录路径前缀有路径
            return replace_md_resource_paths(content, relative_dir)

    def get_source(
        self, environment: "Environment", template: str
    ) -> t.Tuple[str, str, t.Callable[[], bool]]:
        pieces = split_template_path(template)

        for searchpath in self.searchpath:
            # Use posixpath even on Windows to avoid "drive:" or UNC
            # segments breaking out of the search directory.
            filename = posixpath.join(searchpath, *pieces)

            if os.path.isfile(filename):
                break
        else:
            plural = "path" if len(self.searchpath) == 1 else "paths"
            paths_str = ", ".join(repr(p) for p in self.searchpath)
            raise TemplateNotFound(
                template,
                f"{template!r} not found in search {plural}: {paths_str}",
            )

        current_file_path = Path(filename)
        if self.entry_file_path is None:
            self.entry_file_path = current_file_path
        contents = self.process_content(
            self.entry_file_path, current_file_path)

        mtime = os.path.getmtime(filename)

        def uptodate() -> bool:
            try:
                return os.path.getmtime(filename) == mtime
            except OSError:
                return False

        # Use normpath to convert Windows altsep to sep.
        return contents, os.path.normpath(filename), uptodate
